/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************

File        : RTOSInit.c   (for C166/C167 and Keil compiler)
Purpose     : Initializes and handles the hardware for embOS as far
              as required by embOS.
              Feel free to modify this file acc. to your
              target system.
---------------------------END-OF-HEADER------------------------------
*/

#include "RTOS.H"


/*********************************************************************
*
*        Configuration
*
**********************************************************************
*/

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ          1000          /* 1ms Timer Tick */
#endif

#ifdef CPU_C167CR 
  #ifndef OS_FSYS
    #define OS_FSYS             20000000L     /* 20MHz CPU clock */
  #endif
  #ifndef TIMER_PRESCALE
    #define TIMER_PRESCALE      8
  #endif
  #define OS_TIMER_RELOAD       OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ
  #ifndef OS_UART
    #define OS_UART             0
  #endif    
  #ifndef OS_BAUDRATE
    #define OS_BAUDRATE         19200L
  #endif
#else
  #error "Please define a CPU"
#endif

/*********************************************************************
*
*        OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  /* init timer 3:  */
  T3CON = 0x0080;             /* count down */
  T3    = OS_TIMER_RELOAD;    /* load timer 3 */
  T3IC  = 0x0044;             /* interrupt level 1, group level 0 */
  T3R   = 1;                  /* init timer 3 enable register: 1 = running */
  IEN   = 1;                  /* general interrupt enable */
  OS_COM_Init();              /* Initialize UART for embOSView  */
}


/*********************************************************************
*
*        Idle loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not run as task, soit does not have have a stack of its own,
 therefore no functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     /* Idle loop: No task is ready to exec  */
  for (;;);              /* Nothing to do ... wait for interrupt */
}

/*********************************************************************
*
*        Run-time error reaction  (OS_Error)
*
**********************************************************************

 Run-time error reaction

 When this happens, a fatal error has occured and the kernel
 can not continue. In linux, the equivalent would be a
 "kernel panic"

 This routine can be modified to suit your needs ...
 E.g. a red LED could light up. When using an
 emulator, you may set a breakpoint here.
 In the release builds of the library (R), this routine is not required
 (as no checks are performed).
 In the stack check builds (S/SP), only error 120 may occur.
 In the debug builds(D/DP), all of the listed errors may occur.

  The following are the current errorcodes:
  (Please refer to the documentation for more info)
  
OS_ERR_STACK                          (120)
*** invalid or non-initialized data structures ***
OS_ERR_INV_TASK                       (128)
OS_ERR_INV_TIMER                      (129)
OS_ERR_INV_MAILBOX                    (130)
OS_ERR_INV_CSEMA                      (132)
OS_ERR_INV_RSEMA                      (133)

*** Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on a non-1 byte mailbox  ***
OS_ERR_MAILBOX_NOT1                   (135)

***internal errors, please contact SEGGER Microcontrollersysteme ***
OS_ERR_MAILBOX_NOT_IN_LIST            (140)
OS_ERR_TASKLIST_CORRUPT               (142)


*** not matching routine calls or macro usage ***
OS_ERR_UNUSE_BEFORE_USE               (150)
OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151)
OS_ERR_LEAVEINT                       (152)
OS_ERR_DICNT                          (153)
OS_ERR_INTERRUPT_DISABLED             (154)

*** not a legal system call during interrupt ***
OS_ERR_ILLEGAL_IN_ISR                 (160)
*** not a legal system call during timer ***
OS_ERR_ILLEGAL_IN_TIMER               (161)

** double used data structures **
OS_ERR_2USE_TASK                      (170)
OS_ERR_2USE_TIMER                     (171)
OS_ERR_2USE_MAILBOX                   (172)
OS_ERR_2USE_BSEMA                     (173)
OS_ERR_2USE_CSEMA                     (174)
OS_ERR_2USE_RSEMA                     (175)

*/

void OS_Error(int ErrCode) {
  OS_EnterRegion();     // Avoid further task switches
  OS_DICnt =0;          // Allow interrupts so we can communicate
  OS_EI();
  OS_Status = ErrCode;
  while (OS_Status);  
}


/*********************************************************************
*
*        Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in clock cycles.
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt=OS_TIMER_RELOAD-T3;
  OS_U32 time = OS_Time;

  if (T3IC&0x80) {  /* timer interrupt is pending */
    t_cnt=0-T3;
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/OS_TICK_FREQ)*time  + t_cnt;
}


/*********************************************************************
*
*        OS_ConvertCycles2us
*
**********************************************************************

   Convert Cycles into micro seconds. When using 20.0 MHz, this
   means dividing by 20. If you have a different clock frequency,
   you may have to modify this routine in order to get proper
   diagonstics. Please note: Operation of embOS is not affected
   by this routine. It is used for profiling and testing
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;
  return Cycles/(OS_FSYS/1000000);
}


/*********************************************************************
*
*        OS_Tick interrupt
*
**********************************************************************
*/

void OS_ISR_Tick (void) interrupt 0x23 {
  T3 = OS_TIMER_RELOAD;
  OS_TickHandler();
}

#if (OS_UART==0)

/*********************************************************************
*
*        Communication for embOSView   (UART 0)
*
**********************************************************************
*/

  #define BAUDDIVIDE    (OS_FSYS/(32*OS_BAUDRATE))

  sbit P3_10  = P3^10;
  sbit DP3_10 = DP3^10;
  sbit DP3_11 = DP3^11;

  void OS_COM_Init(void) {
    P3_10  = 1;                   /* P3.10-Output-Latch (TxD0) enable    */
    DP3_10 = 1;                   /* P3.10 (TxD0) switch to output       */
    DP3_11 = 0;                   /* switch RxD0 to input                */
    S0BG   = BAUDDIVIDE;          /* baud rate                           */
    S0CON  = 0x80D1;              /* 8N1 + detect framing & overrun      */
    S0EIC  = 0x00;                /* disable error interrupt             */
    S0TIC  = 0x46;                /* TX: Interrupt Level 1 Group Level 2 */
    S0RIC  = 0x45;                /* RX: Interrupt Level 1 Group Level 1 */
  }


  void OS_COM_Send1(OS_U8 c) {
    S0TBUF   = c;                 /* send char */
  }


  void OS_ISR_tx(void) interrupt 0x2a {
    OS_EnterInterrupt();
    OS_OnTx();
    OS_LeaveInterrupt();
  }


  void OS_ISR_rx(void) interrupt 0x2b {
    volatile unsigned char x;

    OS_EnterNestableInterrupt();  /*  We will enable interrupts */ 
    x = S0RBUF;
    if (S0CON & 0x0600) {
      S0CON  =0x80D1;              /* start up Baudrate-Generator, framing+overrun ,       */
    } 
    else {
      OS_OnRx(x);
    }
    OS_LeaveNestableInterrupt();
  }

#elif (OS_UART==1)

/*********************************************************************
*
*        Communication for embOSView   (SIO)
*
**********************************************************************

  SIO does allow synchrone transmission only. Therefore we us it for 
  sending only. embOSView cannot transmit data to target via SIO, but
  it can receive data sent with OS_SendString and display it in the
  Terminal window.
*/

  #define BAUDDIVIDE    (OS_FSYS/(2*OS_BAUDRATE))-1

  sbit P3_9   = P3^9;
  sbit DP3_9  = DP3^9;
  sbit DP3_8  = DP3^8;

  sbit  SSCAREN   = SSCCON^12;

  void OS_COM_Init(void) {

    /* port settings */
    P3_9    = 1;                  /* P3.9 (MTSR) output enable */
    DP3_9   = 1;                  /* P3.9 (MTSR) direction output */
    DP3_8   = 0;                  /* P3.8 (MRST) direction input */ 

    /* Synch. Serial Interface  settings */
    SSCEN   = 0;
    SSCCON  = 0x0009;             /* 10 bit = 8 bit data + start bit + stop bit */
    SSCHB   = 0;                  /* LSB first */
    SSCPH   = 0;                  /* shift data on leading clock */
    SSCPO   = 0;                  /* idle clock low */
    SSCTEN  = 0;                  /* ignore transmit error */
    SSCREN  = 0;                  /* ignore receive error */
    SSCPEN  = 0;                  /* ignore phase error */
    SSCBEN  = 0;                  /* ignore baud rate error */
    SSCAREN = 0;                  /* no action on baud rate error */
    SSCBR   = BAUDDIVIDE;         /* SSCBR = (FSYS/(2*baud))-1 */
    SSCEN   = 1;                  /* enable tx/rx */
    SSCMS   = 1;                  /* master mode */

    /* Interrupt settings */
    SSCRIC  = 0x00;               /* only sending possible */
    SSCEIC  = 0x00;               /* error  interrupt */
    SSCTIC  = 0x45;               /* TX: Interrupt Level 1 Group Level 1 */
  }


  void OS_COM_Send1(OS_U8 c) {
    SSCTB   = ((unsigned int)c<<1) | 0x0200;                 /* send char */
  }

  void OS_ISR_tx(void) interrupt 0x2d {
    OS_EnterInterrupt();
    OS_OnTx();
    OS_LeaveInterrupt();
  }

#else /* No supported UART selected; use dummy routines      */

/*********************************************************************
*
*          Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  c=c;                      /* make compiler happy */
  OS_COM_ClearTxActive();   /* let OS know, that communication finished */
}

#endif

